@@ -1,4 +1,5 @@
 import collections
+import datetime
 import fnmatch
 import json
 import os
@@ -6,7 +7,7 @@ import time
 
 import requests
 import singer
-from singer import metadata, bookmarks, metrics
+from singer import bookmarks, metadata, metrics
 
 session = requests.Session()
 logger = singer.get_logger()
@@ -17,6 +18,7 @@ logger = singer.get_logger()
 REQUIRED_CONFIG_KEYS = ['start_date', 'access_token']
 
 MAX_RATE_LIMIT_WAIT_SECONDS = 600
+RATE_THROTTLING_EXTRA_WAITING_TIME = 60
 
 KEY_PROPERTIES = {
     'commits': ['sha'],
@@ -201,14 +203,23 @@ def calculate_seconds(epoch):
     return int(round((epoch - current), 0))
 
 def rate_throttling(response):
-    if int(response.headers['X-RateLimit-Remaining']) == 0:
-        seconds_to_sleep = calculate_seconds(int(response.headers['X-RateLimit-Reset']))
+    rate_limit_user = response.headers.get('X-RateLimit-Limit', 0)
+    remaining_requests_per_hour = int(response.headers.get('X-RateLimit-Remaining', 0))
+    rate_limit_reset_timestamp = int(response.headers.get('X-RateLimit-Reset', 0))
+    rate_limit_reset_time = datetime.datetime.fromtimestamp(rate_limit_reset_timestamp)
+    if remaining_requests_per_hour == 0:
+        seconds_to_sleep = calculate_seconds(int(rate_limit_reset_timestamp)) + RATE_THROTTLING_EXTRA_WAITING_TIME
 
         if seconds_to_sleep > MAX_RATE_LIMIT_WAIT_SECONDS:
-            message = "API rate limit exceeded, please try after {} seconds.".format(seconds_to_sleep)
+            message = f"API rate limit exceeded: " \
+                      f"User limit per hour: {rate_limit_user}, " \
+                      f"Remaining requests: {remaining_requests_per_hour}, " \
+                      f"Time to reset {rate_limit_reset_time.isoformat()}"
             raise RateLimitExceeded(message) from None
 
-        logger.info("API rate limit exceeded. Tap will retry the data collection after %s seconds.", seconds_to_sleep)
+        logger.warning(f"API user limit rate per hour({rate_limit_user}) exceeded.")
+        logger.warning(f"Time to reset rate limit: {rate_limit_reset_time.isoformat()}")
+        logger.warning(f"Tap will retry data collection after {str(datetime.timedelta(seconds=seconds_to_sleep))}")
         time.sleep(seconds_to_sleep)
 
 # pylint: disable=dangerous-default-value
@@ -258,7 +269,7 @@ def load_schemas():
     for filename in os.listdir(get_abs_path('schemas')):
         path = get_abs_path('schemas') + '/' + filename
         file_raw = filename.replace('.json', '')
-        with open(path) as file:
+        with open(path, encoding='utf-8') as file:
             schemas[file_raw] = json.load(file)
 
     schemas['pr_commits'] = generate_pr_commit_schema(schemas['commits'])
@@ -632,8 +643,6 @@ def get_all_projects(schemas, repo_path, state, mdata, start_date):
 
                 project_id = r.get('id')
 
-
-
                 # sync project_columns if that schema is present (only there if selected)
                 if schemas.get('project_columns'):
                     for project_column_rec in get_all_project_columns(project_id, schemas['project_columns'], repo_path, state, mdata, start_date):
@@ -814,9 +823,7 @@ def get_reviews_for_pr(pr_number, schema, repo_path, state, mdata):
             with singer.Transformer() as transformer:
                 rec = transformer.transform(review, schema, metadata=metadata.to_map(mdata))
             yield rec
-
-
-        return state
+    return state
 
 def get_review_comments_for_pr(pr_number, schema, repo_path, state, mdata):
     for response in authed_get_all_pages(
@@ -829,9 +836,7 @@ def get_review_comments_for_pr(pr_number, schema, repo_path, state, mdata):
             with singer.Transformer() as transformer:
                 rec = transformer.transform(comment, schema, metadata=metadata.to_map(mdata))
             yield rec
-
-
-        return state
+    return state
 
 def get_commits_for_pr(pr_number, pr_id, schema, repo_path, state, mdata):
     for response in authed_get_all_pages(
@@ -849,7 +854,7 @@ def get_commits_for_pr(pr_number, pr_id, schema, repo_path, state, mdata):
                 rec = transformer.transform(commit, schema, metadata=metadata.to_map(mdata))
             yield rec
 
-        return state
+    return state
 
 
 def get_all_assignees(schema, repo_path, state, mdata, _start_date):
@@ -1054,11 +1059,15 @@ def should_skip_repo(
     include_disabled: bool,
 ):
     name = repository.get('name')
+    size = repository.get('size')
     is_archived = repository.get('archived', False)
     is_disabled = repository.get('disabled', False)
     match_excludes = next(filter(lambda exclude: fnmatch.fnmatch(name, exclude), excludes), None)
     match_includes = next(filter(lambda include: fnmatch.fnmatch(name, include), includes), None)
 
+    if size <= 0:
+        return True
+
     if is_archived and not include_archived:
         return True
 
@@ -1220,7 +1229,13 @@ def get_repos_from_config(config) -> list:
     else:
         include_archived = config.get('include_archived', False)
         include_disabled = config.get('include_disabled', False)
-        repos = get_all_repositories(organization, repos_include, repos_exclude, include_archived, include_disabled)
+        repos = get_all_repositories(
+            org=organization,
+            excludes=repos_exclude,
+            includes=repos_include,
+            include_archived=include_archived,
+            include_disabled=include_disabled
+        )
 
         return [r['full_name'] for r in repos]
 
